var path = require('path')
var utils = require('./utils')
var webpack = require('webpack')
var config = require('./config')
var merge = require('webpack-merge')
var baseWebpackConfig = require('./webpack.base.conf')
var CopyWebpackPlugin = require('copy-webpack-plugin')
var OptimizeCSSPlugin = require('optimize-css-assets-webpack-plugin')
var externals = require('./externals')
var ExtractTextPlugin = require('extract-text-webpack-plugin')
var HtmlWebpackPlugin = require('html-webpack-plugin')

var buildOptions = config.dev
var env = buildOptions.env

var webpackConfig = merge(baseWebpackConfig, {
  module: {
    rules: utils.styleLoaders({
      sourceMap: buildOptions.productionSourceMap,
      extract: true
    })
  },

  devtool: buildOptions.productionSourceMap ? '#source-map' : false,

  devServer: {
    contentBase: path.join(__dirname, "../dist"),
    watchContentBase: true,
    port: 9000
  },

  output: {
    path: buildOptions.assetsRoot,
    filename: utils.assetsPath('index.js')
  },

  plugins: [
    new webpack.DefinePlugin({
      'process.env': env
    }),

    // extract css into its own file
    new ExtractTextPlugin({
      filename: utils.assetsPath('css/[name].css')
    }),

    // Compress extracted CSS. We are using this plugin so that possible
    // duplicated CSS from different components can be deduped.
    new OptimizeCSSPlugin({
      cssProcessorOptions: {
        safe: true
      }
    })
  ]
})
module.exports = webpackConfig
