'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.default = function (modulePath) {
  if (!modulePath) throw new Error('异步加载模块的地址为空');

  // 格式化模块地址，自动补全，等等
  modulePath = (0, _pathFormat2.default)(modulePath);

  // 使用 Promise 处理返回结果
  return new _promise2.default(function (resolve, reject) {
    // 根据类型判断创建的标签类型 .js | .css
    if (modulePath.match('[.]js$')) {
      requireScript(resolve, reject, modulePath);
    } else {
      requireCss(modulePath);
    }
  });
};

var _moduleStore = require('./moduleStore');

var _moduleStore2 = _interopRequireDefault(_moduleStore);

var _pathFormat = require('./pathFormat');

var _pathFormat2 = _interopRequireDefault(_pathFormat);

var _exportModuleManager = require('./exportModuleManager');

var _exportModuleManager2 = _interopRequireDefault(_exportModuleManager);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// corss-browser event
function crossBrowserAddEvent(element, event, callback) {
  if (element.addEventListener) {
    // W3C DOM
    element.addEventListener(event, callback, false);
  } else if (element.attachEvent) {
    // IE DOM
    element.attachEvent('on' + event, callback);
  } else {
    // No much to do
    element[event] = callback;
  }
}

function requireScript(resolve, reject, modulePath) {
  // 返回已经缓存过的模块
  if (_moduleStore2.default.has(modulePath)) {
    var module = _moduleStore2.default.get(modulePath);
    resolve(module);
    return;
  }

  /**
  * 使用 script 标签加载异步模块
  * 在异步模块中触发 script 标签的自定义 export 事件返回模块
  **/
  var script = document.createElement('script');

  script.async = true; // 加载完脚本即可执行
  script.src = modulePath;
  script.charset = 'utf-8';
  script.type = 'text/javascript';
  // script.setAttribute('crossorigin', 'anonymous') // 不同域异常信息的抓取

  crossBrowserAddEvent(script, 'load', function (e) {
    var modules = _exportModuleManager2.default.getModules(); // 取脚本中export出的模块
    resolve(modules); // 返回模块
    _moduleStore2.default.set(modulePath, modules); // 缓存模块
  });

  // 请求模块发生请求异常
  crossBrowserAddEvent(script, 'error', function (e) {
    reject(e);
  });

  document.head.appendChild(script);
}

var requireCss = function requireCss(modulePath) {
  // 资源文件只会加载一次
  if (_moduleStore2.default.has(modulePath)) {
    return;
  }

  /**
  * 使用 link 标签加载异步样式文件
  **/
  var link = document.createElement('link');
  link.rel = 'stylesheet';
  link.type = 'text/css';
  link.charset = 'utf-8';
  link.href = modulePath;

  _moduleStore2.default.set(modulePath, true);

  document.head.appendChild(link);
};

module.exports = exports['default'];