import React, {Component} from 'react'
import CascadeItem from './cascade-item'
import httpResolver from '../../properties/http-wrapper/http'
import filterParser, { parseObjValues as filterObjValuesParser } from '../../../helpers/filterParser'

import '../../../../src/components/basis/cascade-select/index.scss'

export default class extends Component {
  constructor(props){
    super(props)
    this.state = {
      ...this.props.selected
    }
  }

  componentDidMount () {
    let { items } = this.props
    let [{ 'key': firstItemKey }] = items

    this.getItemAsyncOptions(firstItemKey)
    
    for (let { key, reflects=[] } of items) {
      if (this.state[key] && this.state[key].value) {
        reflects.forEach(itemKey => {
          setTimeout(() => this.getItemAsyncOptions(itemKey), 0)
        })
      }
    }
  }

  onChange () {
    let resItemsObj = {}
    Object.keys(this.state).forEach(itemKey => {
      let { options, ...usedData } = this.state[itemKey]
      resItemsObj[itemKey] = usedData
    })
    this.props.onChange(resItemsObj)
  }

  clearReflectItems = function _self(itemKey, items) {
    let { reflects=[] } = items.filter(({ key }) => key == itemKey)[0]
    let res = {}
    reflects.forEach(itemKey => {
      res[itemKey] = {}
      Object.assign(res, _self(itemKey, items))
    })
    return res
  }

  onItemChangeHandler = (targetItemKey, selectedItems) => {
    let [ selectedItem={} ] = selectedItems
    let { 'value': selectedItemValue } = selectedItem

    if (!selectedItemValue) {
      let targetItem = Object.assign(this.state[targetItemKey], { 'value': selectedItemValue })
      this.setState({ [targetItemKey]: targetItem })
    } else {
      let item = Object.assign(this.state[targetItemKey], selectedItem)
      this.setState({ [targetItemKey]: item })  
    }

    /* 通过 reflect 激活需要级联的 menu */
    let { items=[] } = this.props
    let { reflects=[] } = items.filter(({ key }) => key == targetItemKey)[0]

    /* 当前选中的节点发生变化，要清理所有当前节点联动的item的数据源 和 选中项 */
    let clearedState = this.clearReflectItems(targetItemKey, items)
    this.setState(clearedState)
    
    /* 选中有效item后，获取联动项的元数据 */
    if (selectedItemValue) {
      reflects.forEach(itemKey => {
        setTimeout(() => this.getItemAsyncOptions(itemKey), 0)
      })  
    }
    
    /* 抛出事件 */
    setTimeout(() => this.onChange(), 0)
  }

  getItemAsyncOptions (itemKey) {
    let { items=[] } = this.props
    let { http=[] } = items.filter(({ key }) => key == itemKey)[0]

    new Promise((reslove, reject) => {
      /* 没有动态数据源，直接返回静态配置 */
      if (!http.length) {
        return reslove([])
      }

      /* 替换请求参数中需要的级联value */
      let httpFirst = http[0]
      let { 'data': firstHttpData={} } = httpFirst
      
      let cascadeSelected = {}
      Object.keys(this.state).forEach(itemKey => {
        cascadeSelected[itemKey] = this.state[itemKey].value
      })

      firstHttpData = filterObjValuesParser(firstHttpData, { cascadeSelected })
      /* copy 数据源，防止重写 */
      httpFirst = Object.assign({}, httpFirst, { 'data': firstHttpData })
      /* request send */
      httpResolver([httpFirst]).then(responses => {
        reslove(responses)
      })
    }).then(responses => {
      let [ selectItems ] = responses
      let { [itemKey]: item={} } = this.state
      item = Object.assign(item, { 'options': selectItems })
      this.setState({ [itemKey]: item })
    })
  }

  render () {
    let { items=[] } = this.props

    return <div className='pg-common-cmp-cascade-select'>
      {
        items.map(({ lable, options=[], key, required=false }) => {
          let { value='', 'options': asyncOptions=[] } = this.state[key] || {}
          options = options.concat(asyncOptions)
          return <CascadeItem
            key={ key }
            onChange={ this.onItemChangeHandler } 
            menuKey={ key }
            lable={ lable }
            selected={ value }
            required={ required }
            options={ options }
          />
        })
      }
    </div>
  }
}