import React, { Component, PropTypes } from 'react'
import { render, findDOMNode } from 'react-dom'
import DropdownList from '@beisen/dropdown-list'

import '../../../../src/components/basis/dropdown-list/index.css'

class CustomDropdownList extends DropdownList {

  componentWillUnmount () {
    this._thisCmpIsUnMounted = true
  }

  onClose = (event) => {

    if (this._thisCmpIsUnMounted) {
      this.CommonMount.unmountBox();
      this.setState({ 'keepActive': false });
      document.removeEventListener('click', this.onClose);
      this.props.onClose && this.props.onClose('close');
      return
    }

    let selfDom = findDOMNode(this);
    let area = document.getElementById("DropdownList_ul");

    if (area && !area.contains(event.target) && !selfDom.contains(event.target)) {
      if(event.target.className.indexOf('sys-icon-close')>-1|| 
        event.target.className.indexOf('u-icon')>-1) {return};
      this.CommonMount.unmountBox();

      this.setState({keepActive: false})
      document.removeEventListener('click', this.onClose);
      this.props.onClose && this.props.onClose("close");
    }

    if (event.target.className == "btn btn_default btn_sm") {
      this.setState({keepActive: false})
    }
  }
}


export default class extends Component {

  onChange = (e, vdom, selected) => {
    let selectedItems = []

    if (Object.prototype.toString.call(selected) === '[object Array]') {
      selectedItems = selected
    } else if (typeof selected === 'object') {
      selectedItems = [selected]
    }

    selectedItems = selectedItems.map(({ text, ...restData }) => {
      return { 'label': text, ...restData }
    })

    this.props.onChange(selectedItems)
  }

  render () {
    let { items=[], selected=[], required=false, multiple=false, disabled=false } = this.props
    let selectedItems = []

    /* 转换Dropdown-list组件标准数据结构 */
    items = items.map(item => {
      let { label, value, ...restData } = item
      let sItemData = {
        value,
        'text': label,
        'isActive': selected.includes(value) ? true : false,
        'isChecked': selected.includes(value) ? true : false,
        ...restData,
      }
      if (selected.includes(value)) selectedItems.push(sItemData)
      return sItemData
    })

    /* 获取显示的内容 */
    let [ firstSelectedItem={} ] = selectedItems
    let defaultValue = firstSelectedItem['text'] || ''

    /* 必填验证 */
    let errorStatus = (required && !selected.length) ? true : false

    return (<div className="pg-common-cmp-dropdown-list">
      <CustomDropdownList
        title={ '' }
        errorStatus={errorStatus}
        errorMsg={'必选'}
        placeholder={ '请选择' }
        defaultValue={ defaultValue }
        openListView={ false }
        multiple={ multiple }
        children={ items }
        required={ false }
        disabled={ disabled }
        onChange={ this.onChange }
      />
    </div>)
  }
}
