import React, { Component } from 'react'
import metadataUtils from '../../../../utils'
import '../../../../../src/components/properties/basis/events-editor/index.css'
import '../../../../../src/components/properties/basis/events-editor/transfer.scss'

export default class extends Component {
  constructor (props) {
    super(props)

    this.state = {
      selectedActiveCmpEvents: {},
      selectedTargetEvent: {},
      selectedActiveCmp: {},
      selectedActiveCmpEvent: {},
      eventParams: {}
    }
  }

  onEventItemSelected (e, events) {
    let newState = Object.assign({}, this.state, { selectedTargetEvent: events })
    this.setState(newState)
    this.props.onChange(newState)
  }

  onActiveCmpEventsItemSelected (e, events) {
    let newState = Object.assign({}, this.state, { selectedActiveCmpEvent: events })
    this.setState(newState)
    this.props.onChange(newState)
  }

  onActiveCmpSelected (e, cmp) {
    let { 'pageState': { cmpsEvents = {} } } = this.props
    let newState = Object.assign({}, this.state, {
      'selectedActiveCmpEvents': cmpsEvents[cmp.classId] || [],
      'selectedActiveCmp': cmp
    })
    this.setState(newState)
    this.props.onChange(newState)
  }

  onEventParamsChange (e) {
    let paramsStr = e.target.value
    let paramsObj = {}
    try {
      paramsObj = JSON.parse(paramsStr)
    } catch (e) {
      console.error('事件参数非标准JSON格式')
    }
    let newState = Object.assign({}, this.state, {
      'eventParams': paramsObj,
    })
    this.setState(newState)
    this.props.onChange(newState)
  }

  render () {
    let { pageState, targetClassId, targetEvents } = this.props

    let { cmpTubStateSetting } = pageState
    let standardPageSource = metadataUtils.toStandardPageSource(cmpTubStateSetting.toJS(), pageState)

    let activeComponents = []
    let { componentList = [], pageSettings = {} } = standardPageSource

    activeComponents.push({
      'id': pageSettings.id,
      'classId': pageSettings.classId,
      'name': pageSettings.name,
      'label': `页面 (${pageSettings.name})`
    })

    for (let { classId, id } of componentList) {
      let metadata = pageState.cmpsMetadatas[classId]
      activeComponents.push({
        id,
        classId,
        'name': metadata.name,
        'label': `${metadata.label} (${metadata.name})`
      })
    }

    let {
      selectedActiveCmpEvents = {},
      selectedTargetEvent = {},
      selectedActiveCmp = {},
      selectedActiveCmpEvent = {},
      eventParams = {}
    } = this.state

    return <div className="pg-events-picker">
      <div className="pg-pep-events-list transfer-list">
        <div className="transfer-list__text">组件可编辑事件</div>
        <div className="transfer-list__text-bottom"></div>
        <div className="transfer-list__body">
          {
            Object.keys(targetEvents).map(eKey => {
              let events = targetEvents[eKey]
              return <div className={ "pg-pep-events-item transfer-list__item" + (selectedTargetEvent.name === eKey ? ' active' : '') }
                onClick={ (e) => { this.onEventItemSelected(e, Object.assign(events, { name: eKey })) } }>
                  <span className="transfer-list__item__text">
                    { events.label } ({ eKey })
                  </span>
              </div>
            })
          }
        </div>
      </div>
      <div className="pg-pep-cmp-picker">
        <div className="pg-pep-cmpp-cmps-list transfer-list">
          <div className="transfer-list__text">当前工作台中初始化的组件</div>
          <div className="transfer-list__text-bottom"></div>
          <div className="transfer-list__body">
            {
              activeComponents.map(cmp => {
                let { classId, name, label, id } = cmp
                return <div className={ "pg-pep-cmpp-item transfer-list__item" + (selectedActiveCmp.id === id ? ' active' : '') }
                  onClick={ (e) => { this.onActiveCmpSelected(e, cmp) } }>
                    <span className="transfer-list__item__text">
                      { label }
                    </span>
                </div>
              })
            }
          </div>
        </div>
        <div className="pg-pep-cmpp-cmps-events-list transfer-list">
          <div className="transfer-list__text">组件可关联的事件</div>
          <div className="transfer-list__text-bottom"></div>
          <div className="transfer-list__body">
            {
              Object.keys(selectedActiveCmpEvents).map(eKey => {
                let events = selectedActiveCmpEvents[eKey]
                return <div className={ "pg-pep-pep-cmpp-events-item transfer-list__item" + (selectedActiveCmpEvent.name === eKey ? ' active' : '') }
                  onClick={ (e) => { this.onActiveCmpEventsItemSelected(e, Object.assign(events, { name: eKey })) } }>
                    <span className="transfer-list__item__text">
                      { events.label } ({ eKey })
                    </span>
                    {
                      (selectedActiveCmpEvent.name === eKey) ?
                        <div className="form-item">
                          <div className="form-item__control form-item__control_has-info">
                          <span className="u-textarea">
                            <textarea
                              className="u-textarea__element pg-pep-event-params-editor"
                              placeholder="事件参数"
                              onChange={ this.onEventParamsChange.bind(this) }>
                                { JSON.stringify(eventParams) }
                            </textarea>
                            <span className="form-item__bottom-line"></span>
                            <span className="u-textarea__tuozhuai-icon">
                              <i className="u-icon pc-sys-tuozhuai-nomal-svg"></i>
                              <i className="u-icon pc-sys-tuozhuai-active-svg"></i>
                            </span>
                          </span>
                          </div>
                        </div>
                      : null
                    }
                </div>
              })
            }
          </div>
        </div>
      </div>
    </div>
  }
}
