import { Component } from 'react'
import CommonPop from '@beisen/common-pop'
import { v1 } from 'uuid'
import '../../../../../../src/components/properties/basis/tab-item-creator/TabItemEditor/index.css'

export default class extends Component {
  constructor (props) {
    super(props)
    let { label, code, value } = this.props
    if (!value) value = v1()
    
    this.state = {
      label,
      code,
      value,
      'labelNone': false,
      'codeNone': false,
      'codeRepeat': false,
      'codeFormat': false,
      'subBtnDisabled':true
    }
  }

  componentDidMount () {
    this.validate()
  }

  onSave () {
    if (!this.validate()) return
    let { code, label, value } = this.state
    this.props.onSaveClickHandler({
      code, label, value
    })
  }

  onCancel () {
    this.props.onCancelClickHandler()
  }

  inputOnChangeClick = (e, inputKey) => {
    this.setState({ [inputKey]: e.target.value })
    setTimeout(() => { this.validate() }, 0)
  }

  validate () {
    let res = true
    let { code='', label='', value='', codeNone, labelNone, codeRepeat, codeFormat } = this.state

    codeNone = false
    labelNone = false
    codeRepeat = false
    codeFormat = false

    if (!code) {
      codeNone = true
      res = false
    }
    if (!label) {
      labelNone = true
      res = false
    }
    if (!code.match(/^[a-zA-Z0-9]+$/)) {
      codeFormat = true
      res = false
    }
    if (this.props.checkCodeRepeat(code, value)) {
      codeRepeat = true
      res = false
    }

    this.setState({ codeNone, labelNone, codeRepeat, codeFormat, subBtnDisabled:!res })
    return res
  }

  render () {

    let { label, code, codeNone, labelNone, codeRepeat, codeFormat, subBtnDisabled } = this.state

    let editorForm = (<div className='ptic-tab-editor'>

      <div className="form-item">
        <div className="form-item__title">
          <label for="" className="form-item__text ptic-require ptic-label">名称</label>
        </div>

        <div className={
          labelNone ? 
          'form-item__control form-item__control_is-active form-item__control_has-info form-item__control_has-error' :
          'form-item__control'
        }>
          <input type="text" className="u-input" placeholder="名称" value={label} onChange={(e) => this.inputOnChangeClick(e, 'label') } />
          <span className="form-item__bottom-line"></span>
          {
            labelNone
              ? 
              <span className="form-item__explain">名称不可为空</span>
              : 
              <span className="form-item__right-icon">
              </span>
          }
        </div>
      </div>

      <div className="form-item">
        <div className="form-item__title">
          <label for="" className="form-item__text ptic-require ptic-label">编码</label>
        </div>

        <div className={
          (codeNone || codeRepeat || codeFormat) ? 
          'form-item__control form-item__control_is-active form-item__control_has-info form-item__control_has-error' :
          'form-item__control'
        }>
          <input type="text" className="u-input" placeholder="编码" value={code} onChange={(e) => this.inputOnChangeClick(e, 'code') } />
          <span className="form-item__bottom-line"></span>
          {
            (codeNone || codeRepeat || codeFormat)
              ?
              <span className="form-item__explain">{ codeRepeat ? '编码重复' : ( codeNone ? '编码不可为空' : '只能输入字母和数字' ) }</span>
              : 
              <span className="form-item__right-icon">
              </span>
          }
        </div>
      </div>

    </div>)
  
    return (
      <CommonPop
        title={ '选项卡' }
        dataType={ 'component' }
        hasFooter={ true }
        hasHeader={ true }
        showMask={ true }
        data={ editorForm }
        submitPop={ ::this.onSave }
        closePop={ ::this.onCancel } 
        subBtnDisabled={ subBtnDisabled }/> : null
    )
  }
}
    