import { Component } from 'react'
import TabItemList from './TabItemList'
import TabItemEditor from './TabItemEditor'
import Select from '../../../basis/select'
import DropdownList from '../../../basis/dropdown-list'
import '../../../../../src/components/properties/basis/tab-item-creator/index.css'
import '../../../../../src/components/properties/basis/tab-item-creator/tab-add-icon.png'

export default class TabControl extends Component {
  constructor (props) {
    super(props)
    this.state = {
      'showViewCreater': false
    }
  }

  componentDidMount () {
    let {
      defaultValueNeedUpdate,
      'attributes': { 'props': { value='' } },
    } = this.props

    if (defaultValueNeedUpdate) {
      this.props.updateSilent({
        'defaultTab': 'is-active-first-item',
        'isActiveFirstItem': true
      })
    }
  }

  defaultTabChangeHander (selectedItems) {
    let defaultTab = ''
    let isActiveFirstItem = false
    
    if (selectedItems.length > 0) {
      defaultTab = selectedItems[0].value
      if (defaultTab == 'is-active-first-item') {
        isActiveFirstItem = true
      }
    }
    let selectedItemValues = selectedItems.map(({ value }) => value)
    let data = Object.assign({}, this.props.data, {
      defaultTab, isActiveFirstItem
    })
    this.props.update(data)
  }

  onTabItemsChangeHandeler (tabItemList) {
    let { 'tabItemList': currentTabItemList=[] } = this.props.data
    let nextItemValues = tabItemList.map(({ value }) => value)
    let data = Object.assign({}, this.props.data, {
      tabItemList
    })
    
    let needRemovedItemCode = undefined
    for (let { 'value': itemValue } of currentTabItemList) {
      if (!nextItemValues.includes(itemValue)) {
        needRemovedItemCode = itemValue
        break /* 一次性只可能删除一个item */
      }
    }
    if (needRemovedItemCode) {
      let { updateAndDisconnect } = this.props
      updateAndDisconnect(needRemovedItemCode, data)
    } else {
      this.props.update(data)
    }
  }

  onEditorAddHandler ({ label, value, code }) {
    let { tabItemList=[], defaultTab } = this.props.data || {}
    tabItemList.push({ label, value, code })

    let data = Object.assign({}, this.props.data, {
      tabItemList,
      'defaultTab': defaultTab == undefined ? value : defaultTab /* 第一次添加的tab默认值 */
    })

    this.setState({ 'showViewCreater': false })
    this.props.update(data)
  }

  onEditorCancelHandler () {
    this.setState({ 'showViewCreater': false })
  }

  onCreateItemHandler () {
    this.setState({ 'showViewCreater': true })
  }

  checkItemCodeRepeat (targetCode, targetValue) {
    let { tabItemList=[] } = this.props.data || {}
    for (let { code, value } of tabItemList) {
      if (targetCode == code && targetValue != value) return true
    }
    return false
  }

  render () {

    let {
      tabItemList=[],
      defaultTab='',
      isActiveFirstItem=false
    } = this.props.data || {}

    let {
      validateStatus={}, validation={},
    } = this.props.attributes.props

    let { showViewCreater } = this.state
    let defaultTabItemList = [{ 'label': '第一个选项卡', 'value': 'is-active-first-item' }].concat(tabItemList)

    /* 默认选项卡选中的节点 */
    let defaultTabSelected = defaultTab ? [defaultTab] : []

    /* 验证 */
    let { 'message': errMsg, valid } = validateStatus

    let hasTabItemRequiredValidate = ('tabItemRequired' in validation)
    let hasDefaultTabRequiredValidate = ('required' in validation)

    let { tabItemRequired={} } = validation
    let { 'required': defaultTabRequired={} } = validation

    return (<div className='pg-props-tab-items-controller'>
      <div className="ptic-item">
        <DropdownList items={ defaultTabItemList } selected={ defaultTabSelected } onChange={ ::this.defaultTabChangeHander } />
        {
          hasDefaultTabRequiredValidate && !defaultTabRequired.valid
          ? <span className="pg-prop-error-text">{ defaultTabRequired.message }</span>
          : null
        }
      </div>
      <div className="ptic-item" style={{ 'margin-top': '6px', 'margin-bottom': '5px' }}>
        <span
          className={ hasTabItemRequiredValidate ? 'pg-prop-common-label pg-prop-common-label-required' : '' }
          style={{ 'margin-bottom': '0' }}
        >
          选项卡
        </span>
        <span onClick={ ::this.onCreateItemHandler } className="ptic-add-icon sys-icon-add"></span>
      </div>
      <div className="ptic-item">
        <TabItemList items={tabItemList} onChange={::this.onTabItemsChangeHandeler} checkCodeRepeat={::this.checkItemCodeRepeat} />
      </div>
      
      {
        hasTabItemRequiredValidate && !tabItemRequired.valid
        ? <div><span className="pg-prop-error-text">{ tabItemRequired.message }</span></div>
        : null
      }

      {
        showViewCreater ?  
          <TabItemEditor
            onSaveClickHandler={::this.onEditorAddHandler} 
            onCancelClickHandler={::this.onEditorCancelHandler} 
            checkCodeRepeat={::this.checkItemCodeRepeat} />
        : null
      }
    </div>)
  }
}
