
import fetch from '../../../../helpers/fetch'
import filterParser, { parseObjValues as filterObjValuesParser } from '../../../../helpers/filterParser'
import * as httpActions from './http-actions'

export default function (http = []) {
  /* 需要做异步请求的配置 */
  return new Promise((resolve, reject) => {
    if (!http.length) resolve([]);

    let httpPromises = []
    http.forEach(httpRequest => {
      let { url, data = {}, method = 'GET', onComplete = [] } = httpRequest
      httpPromises.push(new Promise((resolve, reject) => {        
        fetch(
          {
            method,
            'url': filterParser(url),
            'params': filterObjValuesParser(data),
          },
        ).then(({ 'body': response }) => {
          for (let { type = '', params = {} } of onComplete) {
            let action = httpActions[type]
            if (!action) throw new Error(`Http.onComplete 未找到指定action: ${type}`)
            response = httpActions[type](response, params)
          }
          resolve(response)
        })
      }))
    })

    Promise.all(httpPromises).then(responses => {
      resolve(responses)
    })
  })
}
