import React, {Component, PropTypes} from 'react'
import { connect } from 'react-redux'
import httpResolver from './http'
import filterParser, { parseObjValues as filterObjValuesParser } from '../../../helpers/filterParser'
import PropCmpLabel from '../../basis/prop-cmp-label'
import '../../../../src/components/properties/http-wrapper/index.css'

export default function (WrappedComponent, wrappedOptions = {}) {
  class WrapperComponent extends React.Component {
    constructor (props) {
      super(props)
      this.state = {
        'resources': {},
        'componented': false,
        'defaultValueNeedUpdate': false,  /* 是否需要update默认值 */
      }
    }

    update = (obj) => {
      /* 判断update函数存在，pgBuilder不可控因素 */
      if (this.props.update) {
        this.props.update(obj)
      }
    }

    defaultValueNeedUpdate () {
      let { data } = this.props
      if (data === undefined || data == null) {
        return true
      }
      return false
    }

    componentDidMount () {
      let { http = [] } = this.props._metadata
      httpResolver(http).then(responses => {
        let state = { 'resources': {} }
        responses.forEach(resp => {
          for (let key in resp) {
            state['resources'][key] = resp[key]
          }
        })
        this.setState({
          ...state,
          'componented': true,
          'defaultValueNeedUpdate': this.defaultValueNeedUpdate()
        })
      })
    }

    render () {
      /* 未请求到元数据前不渲染组件 */
      if (!this.state.componented) return null

      let { defaultValueNeedUpdate } = this.state
      let { 'update': $u, _metadata, ...restProps } = this.props

      let { attributes = {}, validation = [], label = '', description = [] } = _metadata
      let { props = {}, ...restAttrs } = attributes

      restProps = {
        ...restProps,
        /* 把动态数据源merge进props, 传入属性组件 */
        'attributes': {
          'props': {
            ...filterObjValuesParser(props), /* 转换props中的每一个keyValue */
            ...this.state.resources,
            label,
            validation,
            description,
          },
          ...restAttrs,
        },
        defaultValueNeedUpdate,
      }

      return <div className="pg-prop-wrapper">
        <PropCmpLabel label={ label } validation={ validation } description={ description } />
        <WrappedComponent httpResolver={httpResolver} { ...restProps } update={ this.update } _metadata={_metadata} />
      </div>
    }
  }
  return WrapperComponent
}
