import Text from './basis/text'
import Label from './basis/label'
import Textarea from './basis/textarea'
import Checkbox from './basis/checkbox'
import CheckboxList from './basis/checkbox-list'
import Select from './basis/select'
import SelectList from './basis/select-list'
import RichEditorText from './basis/rich-editor-text'
import JsonEditor from './basis/json-editor'
import CascadeSelector from './basis/cascade-selector'
import TabItemCreator from './basis/tab-item-creator'
import EventsEditor from './basis/events-editor'

/* 高阶组件 */
import httpWrapper from './http-wrapper'
import validationWrapper from './validate-wrapper'
import pageStateWrapper from './page-state-wrapper'

let properties = {
  Text,
  Textarea,
  CheckboxList,
  Label,
  Select,
  SelectList,
  Checkbox,
  RichEditorText,
  JsonEditor,
  CascadeSelector,
  TabItemCreator,
  EventsEditor
}

export default properties

/*
* 遍历每一个基础组件，包裹高阶组件
* httpWrapper：处理请求
* validationWrapper 处理验证
*/
Object.keys(properties).forEach(name => {
  let cmp = properties[name]
  properties[name] = httpWrapper(validationWrapper(pageStateWrapper(cmp)))
})

export const installProperties = (insProperties) => {
  Object.keys(insProperties).forEach(name => {
  	properties[name] = httpWrapper(validationWrapper(pageStateWrapper(insProperties[name])))
  })  
}
