import React, { Component } from 'react'
import basedValidates from './validation'
import filterParser, { parseObjValues as filterObjValuesParser } from '../../../helpers/filterParser'
import '../../../../src/components/properties/validate-wrapper/index.css'

export default Target => class extends Component {
  constructor (props) {
    super(props)
    this.state = {}
  }

  verify () {
    let verifiedList = []
    let { data } = this.props
    let { 'props': { validation = [] } } = this.props.attributes

    /* 执行配置的验证规则 */
    for (let { type, message, params = {} } of validation) {
      if (basedValidates[type]) {
        let valid = basedValidates[type](data, params, { filterParser, filterObjValuesParser })
        verifiedList.push({ type, message, params, valid })
      } else {
        throw new Error(`未找到指定的验证规则: ${type}`)
      }
    }
    return verifiedList
  }

  render () {
    let validation = this.verify()
    let { attributes={}, ...restProps } = this.props

    let validateStatus = { 'valid': true }
    for (let { type, message, params, valid } of validation) {
      if (!valid) {
        validateStatus = { type, message, params, valid }
        break
      }
    }

    let validationMap = {}
    validation.forEach(({ type, ...restData}) => { validationMap[type] = { ...restData } })
    Object.assign(attributes.props, { validateStatus, 'validation': validationMap })

    return <div>
      <Target { ...restProps } attributes={ attributes } />
    </div>
  }
}
