import { polyfill } from 'es6-promise'
import 'isomorphic-fetch'
var URLSearchParams = require('url-search-params')

export default function (request) {
  let { url='', method='GET', params={}, headers={}, dataType, mark } = request
  method = method.toUpperCase()

  /* 编辑 fetch API 需要的请求参数 */
  request = {
    'credentials': 'include',
    'headers': {
      'Accept': 'application/json, application/xml, text/play, text/html, *.*',
      'Content-Type': 'application/json; charset=utf-8'
    },
    'method': method
  }

  /* 序列化 GET 请求的 url */
  if (method === 'GET') {
    url += (url.indexOf("?") != -1 ? '&' : '?') + (function(obj) {
      var result = [];
      for (var k in obj) {
        result.push(encodeURIComponent(k) + '=' + encodeURIComponent(obj[k]))
      }
      return result.join('&')
    })(params);
  } else if (headers && headers['Content-Type'] == 'application/x-www-form-urlencoded') {
    /* 序列化表单请求参数 */
    var urlSear = new URLSearchParams();
    for(var i in params) {  
      urlSear.append(i, params[i]);
    }
    request.body = urlSear.toString()
  } else {
    request.body = JSON.stringify(params)
  }
  
  /*
  * 发送请求
  * @dataType: 根据 dataType 转化接口返回对象
  */
  return fetch(url, request)
    .then(response => {
      let formatApi = dataType == 'text' ? 'text' : 'json'
      if (!response.ok) {
        response[formatApi] = function() {
          return this.text().then(function(data) {
            try {
              return JSON.parse(data);
            } catch(e) {
              return Promise.reject(response);
            }
          })
        }
      }
      return response[formatApi]().then(json => ({ json, response }))
    })
    .then(({ json, response }) => {
      if (!response.ok) {
        return Promise.reject({ json, response });
      }
      /* 返回标准Request对象 */
      return {
        'ok': response.ok,
        'status': response.status,
        'statusText': response.statusText,
        'headers': response.headers,
        'bodyUsed': response.bodyUsed,
        'redirected': response.redirected,
        'type': response.type,
        'url': response.url,
        'body': json
      }
    })
}
