import asyncRequire from './asyncRequire'

/*
* 如果传入字符串，则使用单个模块加载模式
*/
var singleModuleImport = function (modulePath) {
  return asyncRequire(modulePath)
}

/*
* 如果传入数组，则使用多个模块加载模式
*/
var multiModuleImport = function (modulePaths) {
  return new Promise(function (resolve, reject) {
    // 创建模块异步加载的 promise 对象
    var moduleAsyncHandlers = []
    for (let i = 0; i < modulePaths.length; i++) {
      moduleAsyncHandlers.push(
        asyncRequire(modulePaths[i])
      )
    }

    // 加载所有的模块
    Promise.all(moduleAsyncHandlers)
      .then(function (modules) {
        resolve(modules)
      })
      .catch(function (error) {
        reject(error)
      })
  })
}

// 异步请求组件的函数
export default function (modulePaths) {
  // 兼容请求单个或者多个模块
  if (typeof modulePaths === 'string') return singleModuleImport(modulePaths)
  else if (Object.prototype.toString.call(modulePaths) === '[object Array]') return multiModuleImport(modulePaths)
  else throw new Error('模块地址参数格式错误，字符串或者数组')
}
