import React, { Component } from 'react'
import GridWorkspace, { TubState } from '@beisen/grid-page-builder'
import metadataUtils from './utils'

import pageTemplates, { installPageTemplates } from './components/page-templates'
import components, { installElements } from './components/elements'
import properties, { installProperties } from './components/properties'
import validationFuns, { injectValidatesScope, installValidates } from './components/properties/validate-wrapper/validation'
import { installState } from './components/properties/page-state-wrapper'

import { installFormatters } from './formatters'
import { installContexts } from './contexts'
import { injectContextsScope } from './contexts'
import metadataParser from './metadataParser'

import filterParser, { parseObjValues as filterObjValuesParser } from './helpers/filterParser'

import immutable from 'immutable'
import 'react-transition-group'

import { v1 } from 'uuid'
import '../src/static/css/index.css'

import 'babel-polyfill'
import 'react-addons-css-transition-group'
import '@beisen/nade-style/style.css'
import '@beisen/grid-page-builder/style.css'
import '../src/static/css/nature.design.min.css'
// import '../src/static/images/index.js'


let { toPBuilderPageSource, toStandardPageSource, transferToPgbEditProps, transferToPgbCmps } = metadataUtils

export default class extends Component {
  constructor (props) {
    super(props)
    let {
      elementComponents = [],
      pageComponent = {},
      pageSource: standardPageSource = {} } = props

    let pageSource = toPBuilderPageSource(standardPageSource)
    let tubSetting = TubState.create(pageSource)

    tubSetting.setSavedState(this.props.saveBtnStatus)

    this.state = {
      'cmpTubStateSetting': tubSetting || {},
      'pageComponent': pageComponent,
      'pageSource': pageSource,
      'standardPageSource': standardPageSource,
      'pageSource.ComponentList.classIds.mapping': {},
      'elementComponents': elementComponents,
      'saveBtnStatus': false,
      'cmpsMetadatas': {},
      'cmpsEditProps': {},
      'cmpsEvents': {},
      'hooks': {
        'instanceTimes': {},
        'validation': {},
        'onSubmit': {}
      }
    }

    /* 提取组件元数据中的hooks和可以编辑属性 写入store */
    for (let groupMetadata of this.state.elementComponents) {
      for (let cmpMetadata of groupMetadata.components) {
        /* start */
        this.attachEventsToSpecProperty(
          this.cacheCmpsValidation(
            this.cacheCmpsMetadata(
              this.cacheCmpsEvents(
                this.cacheCmpsHooksToStore(
                  this.cacheCmpsEditProps(cmpMetadata)
                )
              )
            )
          )
        )
        /* end */
      }
    }

    /* 页面组件的扁平配置 */
    this.attachEventsToSpecProperty(
      this.cacheCmpsValidation(
        this.cacheCmpsMetadata(
          this.cacheCmpsEvents(
            this.cacheCmpsHooksToStore(
              this.cacheCmpsEditProps(this.state.pageComponent)
            )
          )
        )
      )
    )

    /*
    * 创建pageSource.ComponentList中组件id和classId的映射关系
    * pgb编辑态的时候，getEditProps API 未返回classId，找不到对应的props
    */
    this.makePageSourceCmpClassIds()

    /*
    * 把state注入属性组件，提供使用
    */
    installState(this.state)

    /*
    * 属性组件验证函数需要全部的state用来做验证
    * girdpagebuilder组件无法控制组件所有状态，通过外部包裹函数的手段添加state
    */
    installValidates(this.props.validates)
    injectValidatesScope(this.state)

    /* 注入scope进contexts */
    injectContextsScope(this.state)

    /*
    * 提供给外部注入
    */
    installPageTemplates(this.props.pageTemplates)
    installElements(this.props.elements)
    installProperties(this.props.properties)
    installFormatters(this.props.formatters)
    installContexts(this.props.contexts)
  }

  /*
  * 创建pageSource.ComponentList中组件id和classId的映射关系
  * pgb编辑态的时候，getEditProps API 未返回classId，找不到对应的props
  */
  makePageSourceCmpClassIds () {
    let { componentList = [] } = this.state.pageSource
    for (let { id, classId } of componentList) {
      this.state['pageSource.ComponentList.classIds.mapping'][id] = classId
    }
  }

  getInstanceTimes (id) {
    return this.state.hooks.instanceTimes[id]
  }

  getValidation (id) {
    return this.state.hooks.validation[id]
  }

  cacheCmpsMetadata (cmpMetadata = {}) {
    let { id } = cmpMetadata
    this.state.cmpsMetadatas[id] = cmpMetadata
    return cmpMetadata
  }

  cacheCmpsEvents (cmpMetadata = {}) {
    let { id, events } = cmpMetadata
    this.state.cmpsEvents[id] = events
    return cmpMetadata
  }

  cacheCmpsValidation (cmpMetadata = {}) {
    let { 'id': cmpId, editProps = [] } = cmpMetadata
    for (let { 'name': propName, validation } of editProps) {
      if (!this.state.hooks.validation[cmpId]) this.state.hooks.validation[cmpId] = {}
      Object.assign(this.state.hooks.validation[cmpId], {
        [propName]: validation
      })
    }
    return cmpMetadata
  }

  /*
  * 把组件元数据中配置events添加到对应的事件编辑属性组件中
  * 在对应属性组件中，需要获取到当前属性的所属组件的events，和全局其他组件的events
  */
  attachEventsToSpecProperty (cmpMetadata = {}) {
    let specPropertyType = 'events-editor'
    let { id, events = {}, editProps = [] } = cmpMetadata
    for (let prop of editProps) {
      if (prop.type === specPropertyType) {
        let { attributes = {} } = prop
        let props = Object.assign(attributes.props || {}, {
          targetClassId: id,
          targetEvents: events
        })
        Object.assign(attributes, { props })
        Object.assign(prop, { attributes })
      }
    }
    return cmpMetadata
  }

  /*
  * 注册的组件的元数据中接受固定的几个参数
  * 非标准的参数将会被用于自定义处理，比如目前正在使用的: onSubmit, instanceTimes, beforeSave
  */
  cacheCmpsHooksToStore (cmpMetadata = {}) {
    let { name, type, label, id, data, editProps, ...resthooks } = cmpMetadata
    // 处理元数据中配置的hooks, 创建对应的扁平数据结构
    Object.keys(resthooks).forEach(hookName => {
      if (!this.state.hooks[hookName]) this.state.hooks[hookName] = {}
      Object.assign(this.state.hooks[hookName], {
        [id]: resthooks[hookName]
      })
    })
    return cmpMetadata
  }

  cacheCmpsEditProps (cmpMetadata = {}) {
    let { id, editProps = {} } = cmpMetadata
    this.state.cmpsEditProps[id] = editProps
    return cmpMetadata
  }

  componentDidMount () {
    this.state.cmpTubStateSetting.setSavedState(false)
  }

  /* 情况特殊，直接通过callback返回数据源，不走单项数据流 */
  /* pageBuilder 对每个组件props缓存，只会调用一次 */
  getEditProps = (type, id) => {
    return new Promise((resolve, reject) => {
      let stardandEditProps = this.state.cmpsEditProps[id]
      /*
      * 创建pageSource.ComponentList中组件id和classId的映射关系
      * pgb编辑态的时候，getEditProps API 未返回classId，找不到对应的props
      */
      if (!stardandEditProps) {
        let mappedPageSourceClassid = this.state['pageSource.ComponentList.classIds.mapping'][id]
        stardandEditProps = this.state.cmpsEditProps[mappedPageSourceClassid]
      }
      let pgbEditProps = transferToPgbEditProps(stardandEditProps)
      resolve(pgbEditProps)
    })
  }

  handleChange = (cmpTubStateSetting) => {
    let standardPageSource = toStandardPageSource(cmpTubStateSetting.toJS(), this.state)
    this.setState({ cmpTubStateSetting, standardPageSource })
    this.props.onChange(standardPageSource)
  }

  handleSave = (tubStateSource) => {
    let activeCmp = this.validateOnSave(tubStateSource)
    if (activeCmp) {
      this.setState({
        'cmpTubStateSetting': this.state.cmpTubStateSetting.setSelect(activeCmp.id)
      })
    } else {
      let standardPageSource = toStandardPageSource(tubStateSource, this.state)
      let metadata = metadataParser(standardPageSource, this.state)
      this.props.savePage(metadata)
    }
  }

  genId () {
    return v1()
  }

  /*
  * 验证组件中的可编辑属性，通过配置的validation
  */
  validateCmp (editableData, validateCmpConfig = {}, cmpId) {
    for (let edKey in editableData) {
      if (validateCmpConfig[edKey]) {
        let propValidation = validateCmpConfig[edKey]
        for (let { type, message, params } of propValidation) {
          let data = editableData[edKey]
          if (validationFuns[type]) {
            let valiRes = validationFuns[type](data, params, { filterParser, filterObjValuesParser })
            if (!valiRes) {
              return { id: cmpId }
            }
          } else {
            throw new Error(`未找到指定验证函数: ${type}`)
          }
        }
      }
    }
  }

  validateOnSave (tubState) {
    let { componentList = [], pageSettings = {} } = tubState

    /* 组件的验证 */
    for (let { editableData, name, id, classId } of componentList) {
      /*
      * classId: 和左侧拖拽组件对应的 id 属性
      * pgb左侧的组件都会有一个id
      * 组件被拖拽到控制台，在控制台中组件会生成一个新的id（实例id），classId为左侧属性组件的id
      */
      let res = this.validateCmp(editableData, this.getValidation(classId), id)
      if (res) return res
    }

    /* 页面组件的验证 */
    return this.validateCmp(
      pageSettings.editableData,
      this.getValidation(this.state.pageComponent.id),
      null
    )
  }

  onBeforeCreateComponent = ({ name, type, id }, pageData) => {
    /* 在 组件和实例化次数 mapping 关系中，找到当前组件的初始化限制 */
    let instanceTimes = this.getInstanceTimes(id)
    /* 获取cmp在视图中已经被实例化的次数 */
    let instanceCmpId = (pageData.componentList.filter(item => item.cType == name)).length;
    /* 未曾被实例化过 */
    if (!instanceCmpId) {
      return true;
    } else {
      let limit = (() => {
        let pageType = this.state.pageComponent.type
        for (let insKey in instanceTimes) {
          if (insKey.toLowerCase() === pageType.toLowerCase()) {
            return instanceTimes[insKey]
          }
        }
        /* 未配置 instanceTimes，不做限制 */
        return 9999
      })()
      return instanceCmpId < limit;
    }
  }

  onBeforeRemoveComponent = ({ cType }) => {
    // this.instance[cType] = this.instance[cType] - 1
  }

  getPageTemplate () {
    let PageTemplate = pageTemplates[this.state.pageComponent.type]
    if (!PageTemplate) {
      throw new Error(`未找到页面组件: ${this.state.pageComponent.type}`)
    }

    /* 向布局模板组件注入 属性配置 */
    let stardandEditProps = this.state.pageComponent.editProps
    let pbgEditProps = transferToPgbEditProps(stardandEditProps)
    PageTemplate.editProps = pbgEditProps
    return PageTemplate
  }

  componentWillReceiveProps (nextProps) {
    if (nextProps.saveBtnStatus !== this.state.saveBtnStatus) {
      this.state.cmpTubStateSetting.setSavedState(nextProps.saveBtnStatus)
    }
  }

  render () {
    // previewUrl = '#/gview' 
    return (<div>
      <GridWorkspace
        tubState = {this.state.cmpTubStateSetting}
        template = {this.getPageTemplate()}
        components = {components}
        defaultProps = {{ 'component': [] }}
        propsComponents = {properties}
        availableComponents = {transferToPgbCmps(this.state.elementComponents)}
        onChange = {this.handleChange}
        onSave = {this.handleSave}
        genUID = {this.genId}
        onBeforeCreateComponent = {this.onBeforeCreateComponent}
        onBeforeRemoveComponent = {this.onBeforeRemoveComponent}
        onMessage = {this.handleMessage}
        getEditProps = {this.getEditProps}
      />
    </div>)
  }
}
