import ValidateError from './ValidateError'
import tabContainerDynamic from './tabContainerDynamic'
import makeDataSource from './makeDataSource'
import validateRequiredCmps from './validateRequiredCmps'

const hooks = {
  tabContainerDynamic,
  makeDataSource,
  validateRequiredCmps,
}

let executeHook = (tubState, cmp = {}, configs = []) => {
  for (let { type = '', params = {} } of configs) {
    if (!hooks[type]) {
      throw new Error(`未找到指定 onSubmit 函数: ${type}`)
    }
    tubState = hooks[type](tubState, cmp, params)
  }
  return tubState
}

let onSubmit = (tubState, state) => {
  let cmpConfigs = state.hooks.onSubmit
  let pageSettingConfigs = state.hooks.onSubmit[state.pageComponent.id]

  /* pageSetting */
  tubState = executeHook(tubState, tubState.pageSettings, pageSettingConfigs)

  /* cmps */
  let { componentList = [] } = tubState
  componentList.map(({ id, name }) => ({ name, id })).forEach(({ 'name': cmpName, 'id': cmpId }) => {
    let { [cmpId]: configs=[] } = cmpConfigs
    let { 'componentList': cmps=[] } = tubState
    let cmp = cmps.filter(({ id }) => id == cmpId)[0]
    tubState = executeHook(tubState, cmp, configs)
  })

  return tubState
}


export default (tubState, state, next) => {
  try {
    let newTubState = onSubmit(tubState, state)
    return next(newTubState)
  } catch (e) {
    if (e instanceof ValidateError) {
      alert(e.message)
      return false
    } else {
      throw e
    }
  }
}
